/*
 * Licensed Materials - Property of IBM
 *
 * (C) Copyright IBM Corp. 2011 All Rights Reserved
 *
 * The source code for this program is not published or other-
 * wise divested of its trade secrets, irrespective of what has
 * been deposited with the U.S. Copyright Office.
 *
 * DISCLAIMER:
 * This program contains code made available by IBM Corporation on an
 * "AS-IS" basis. This sample code has not been thoroughly tested under all conditions. 
 * IBM, therefore, cannot guarantee or imply reliability, serviceability, or function. 
 * IBM provides no program services for this material. 
 * 
 * THIS MATERIAL IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESS OR 
 * IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF FITNESS FOR A 
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SOME JURISDICTIONS DO NOT ALLOW THE 
 * EXCLUSION OF IMPLIED WARRANTIES, SO THE ABOVE EXCLUSIONS MAY NOT APPLY TO YOU. 
 * IN NO EVENT WILL IBM BE LIABLE TO ANY PARTY FOR ANY DIRECT, INDIRECT, SPECIAL OR OTHER 
 * CONSEQUENTIAL DAMAGES FOR ANY USE OF THIS MATERIAL INCLUDING, WITHOUT LIMITATION, ANY 
 * LOST PROFITS, BUSINESS INTERRUPTION, LOSS OF PROGRAMS OR OTHER DATA ON YOUR INFORMATION
 * HANDLING SYSTEM OR OTHERWISE, EVEN IF EXPRESSLY ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
 * 
 * You may copy and modify this source code and may redistribute modified versions of this 
 * source code, provided however that you do not alter or delete any copyright information  
 * or notices contained in the source code. 
 * 
 */
package com.ibm.are.example;

import java.util.HashMap;

import com.ibm.are.common.Version;
import com.ibm.are.platform.impl.IBMi;
import com.ibm.are.plugin.BasePlugin;
import com.ibm.are.plugin.ResultInfo;
import com.ibm.are.xml.rules.config.ConfigurationRule;
import com.ibm.are.xml.rules.fileattr.FileTypeRule;
import com.ibm.ibmi.are.xml.rules.authority.DataAuthorityRule;
import com.ibm.ibmi.are.xml.rules.authority.ObjectAuthorizationListRule;
import com.ibm.ibmi.are.xml.rules.authority.ObjectOwnerRule;
import com.ibm.ibmi.are.xml.rules.other.HostNameRule;
import com.ibm.ibmi.are.xml.rules.other.SystemValueRule;
import com.ibm.ibmi.are.xml.rules.product.ProductStatusRule;
import com.ibm.ibmi.are.xml.rules.userprofile.GroupProfileRule;
import com.ibm.ibmi.are.xml.rules.userprofile.OwnerRule;

// IMPORTANT:  This plugin requires ARE Core version 1.6.6 or newer to run correctly
public class ApiUsageExamples extends BasePlugin {

	protected Version initPluginVersion() {
		return new Version(1, 0, 0);
	}

	// IMPORTANT:  This plugin requires ARE Core version 1.6.6 or newer to run correctly
	protected ResultInfo runImpl() {
		//
		// Configuration File
		//
		// Populate a HashMap with the information necessary to check a value in
		// a property file.  We then provide this information to the configuration
		// rule object using its API interface.
		HashMap configFileRuleInfo = new HashMap();
		configFileRuleInfo.put("name", "/QIBM/ProdData/OS/OSGi/LWI81/conf/config.properties");	// Full path to config file
		configFileRuleInfo.put("type", "propertyFile");											// The type of config file
		configFileRuleInfo.put("key", "osgi.configuration.cascaded");							// The key whose value we want to verify
		configFileRuleInfo.put("expectedValue", "false");										// The expected value for the key
		reportStep("Checking property configuration file");
		ConfigurationRule propCheck = new ConfigurationRule(configFileRuleInfo);

		// Invoking the run() method is how we actually perform the check.  Note that
		// the rule itself takes care of reporting the result of the check.
		propCheck.run(this);

		
		//
		// Configuration File
		//
		HashMap configFileRuleInfo2 = new HashMap();
		configFileRuleInfo2.put("name", "/QIBM/ProdData/OS/OSGi/LWI81/conf/config.properties");	// Full path to config file
		configFileRuleInfo2.put("type", "propertyFile");											// The type of config file
		configFileRuleInfo2.put("key", "osgi.startLevel");										// The key whose value we want to verify
		configFileRuleInfo2.put("expectedValue", "20");											// The expected value for the key
		configFileRuleInfo2.put("checkClass", "numeric");										// Treat expected/actual values as numbers
		configFileRuleInfo2.put("checkType", "lessThan");										// Verify actual value is less than expected value 
		reportStep("Checking property configuration file again");
		ConfigurationRule propCheck2 = new ConfigurationRule(configFileRuleInfo2);
		propCheck2.run(this);

		
		//
		// User Profile
		//
		HashMap userProfileRuleInfo = new HashMap();
		userProfileRuleInfo.put("name", "QLWISVR");										// The IBM i user profile to verify 
		userProfileRuleInfo.put("expectedValue", "*NONE");								// The expected value for the group profile
		reportStep("Checking the user profile's group profile attribute");
		GroupProfileRule userProfileCheck = new GroupProfileRule(userProfileRuleInfo);
		userProfileCheck.run(this);

		
		//
		// User Profile
		//
		HashMap userProfileRuleInfo2 = new HashMap();
		userProfileRuleInfo2.put("name", "QLWISVR");					// The IBM i user profile to verify 

		// We want to check the owner of the user profile.  In this case a value of
		// either *USRPRF or QLWISVR is fine.  So we list both items in the standard
		// StringList list format, and specify a check type of "in", which means the
		// check will ensure the group profile equals *any* value in the list we specified.
		userProfileRuleInfo2.put("expectedValue", "['*USRPRF' 'QLWISVR']");
		userProfileRuleInfo2.put("checkType", "in");
		reportStep("Checking the user profile's owner attribute");
		OwnerRule userProfileCheck2 = new OwnerRule(userProfileRuleInfo2);
		userProfileCheck2.run(this);

		
		//
		// TCP/IP Host Table Entries
		//
		HashMap hostTableRuleInfo = new HashMap();
		hostTableRuleInfo.put("expectedValue", "localhost");		// Looking for LOCALHOST in the host table

		// Using the 'contains' check type means the host table can contain other
		// host names as well.  For the check to succeed, the host table must contain
		// the expected value (LOCALHOST) but it can also contain other values.
		hostTableRuleInfo.put("checkType", "contains");
		reportStep("Checking the TCP/IP host table");
		HostNameRule hostNameCheck = new HostNameRule(hostTableRuleInfo);
		hostNameCheck.run(this);

		
		//
		// Authority Attributes
		//
		HashMap authorityRuleInfo = new HashMap();
		authorityRuleInfo.put("name", "/QIBM/ProdData/OS/OSGi/LWI81");		// Directory we want to check the authority of
		authorityRuleInfo.put("user", "*PUBLIC");							// Checking authority of *PUBLIC
		authorityRuleInfo.put("dataAuth", "*RX");							// Checking that *PUBLIC has *RX authority to the directory
		reportStep("Checking *PUBLIC authority to a directory");
		DataAuthorityRule dataAuthCheck = new DataAuthorityRule(authorityRuleInfo);
		dataAuthCheck.run(this);


		//
		// Authority Attributes
		//
		HashMap authorityRuleInfo2 = new HashMap();
		authorityRuleInfo2.put("name", "/QSYS.LIB/QHTTPSVR.LIB/QHTTPMSG.MSGF"); // Checking the owner of the HTTP message file
		authorityRuleInfo2.put("owner", "QSYS");							// Checking that the owner is QSYS 
		reportStep("Checking the owner of the HTTP message file");
		ObjectOwnerRule ownerCheck = new ObjectOwnerRule(authorityRuleInfo2);
		ownerCheck.run(this);

		
		//
		// Authority Attributes
		//
		HashMap authorityRuleInfo3 = new HashMap();
		authorityRuleInfo3.put("name", "/QIBM/ProdData/OS/OSGi/LWI81/version.properties"); // Checking the owner of the HTTP message file
		authorityRuleInfo3.put("authList", "INCORRECT_VALUE");					// Check the authorization list against some incorrect value 
		reportStep("Checking the authorization list of a file using different severity levels");
		ObjectAuthorizationListRule authListCheckError = new ObjectAuthorizationListRule(authorityRuleInfo3);
		authListCheckError.run(this);	// This will result in a problem with an Error severity level being reported

		// Change severity level to warning and check again
		authorityRuleInfo3.put("errorLevel", "warning"); 
		ObjectAuthorizationListRule authListCheckWarning = new ObjectAuthorizationListRule(authorityRuleInfo3);
		authListCheckWarning.run(this);	// This will result in a problem with a Warning severity level being reported

		// Change severity level to warning and check again
		authorityRuleInfo3.put("errorLevel", "info"); 
		ObjectAuthorizationListRule authListCheckInfo = new ObjectAuthorizationListRule(authorityRuleInfo3);
		authListCheckInfo.run(this);	// This will result in a problem with an Info severity level being reported

		
		//
		// File Attributes
		//
		HashMap fileAttrRuleInfo = new HashMap();
		fileAttrRuleInfo.put("name", "/QIBM/ProdData/OS/OSGi/LWI81/version.properties"); // Checking the owner of the HTTP message file
		fileAttrRuleInfo.put("type", "FILE"); 			// We expect this to be a real file (not a dir or symlink)
		reportStep("Checking the file type");
		FileTypeRule typeRule = new FileTypeRule(fileAttrRuleInfo);
		typeRule.run(this);

		
		//
		// IBM i Products
		//
		HashMap productRuleInfo = new HashMap();
		
		// Get the product ID prefix for the IBM i OS we are running the verification on.
		// If we are running on IBM i 6.1, this method will return 5761.  If we are running
		// on IBM i 7.1, it will return 5770.
		String idPrefix = IBMi.get().getProductIdPrefix();
		productRuleInfo.put("name", idPrefix + "SS1"); 		// Checking a base operating system product
		productRuleInfo.put("option", "30"); 				// Option 30 of SS1 is QShell
		productRuleInfo.put("status", "installed");			// We expect QShell to be installed
		reportStep("Checking the install status of QShell");
		ProductStatusRule prodStatusRule = new ProductStatusRule(productRuleInfo);
		prodStatusRule.run(this);


		//
		// System Values
		//
		HashMap sysValRuleInfo = new HashMap();
		sysValRuleInfo.put("name", "QCCSID"); 		// Checking the owner of the HTTP message file

		// Check to make sure QCCSID is NOT set to 65535
		sysValRuleInfo.put("expectedValue", "65535");
		sysValRuleInfo.put("checkType", "doesNotEqual");
		reportStep("Checking a system value");
		SystemValueRule sysValRule = new SystemValueRule(sysValRuleInfo);
		sysValRule.run(this);

		// We are done running and did so successfully
		return new ResultInfo(true);
	}

	public String getCommonName() {
		return "Rule class API usage examples";
	}

	public String getDescription() {
		return "Code examples of using the Rule class API interface to leverage those classes for verifying application and system attributes.";
	}
}
